@@ -21,8 +21,33 @@
 #include "regdb.h"
 #include "reglib.h"
 
+#ifndef CONFIG_LIBNL20
+/* libnl 2.0 compatibility code */
+static inline struct nl_handle *nl_socket_alloc(void)
+{
+       return nl_handle_alloc();
+}
+
+static inline void nl_socket_free(struct nl_handle *h)
+{
+       nl_handle_destroy(h);
+}
+
+static inline int __genl_ctrl_alloc_cache(struct nl_handle *h, struct nl_cache **cache)
+{
+       struct nl_cache *tmp = genl_ctrl_alloc_cache(h);
+       if (!tmp)
+               return -ENOMEM;
+       *cache = tmp;
+       return 0;
+}
+
+#define genl_ctrl_alloc_cache __genl_ctrl_alloc_cache
+#define nl_sock nl_handle
+#endif /* CONFIG_LIBNL20 */
+
 struct nl80211_state {
-	struct nl_handle *nl_handle;
+	struct nl_sock *nl_sock;
 	struct nl_cache *nl_cache;
 	struct genl_family *nl80211;
 };
@@ -31,23 +56,22 @@ static int nl80211_init(struct nl80211_state *state)
 {
 	int err;
 
-	state->nl_handle = nl_handle_alloc();
-	if (!state->nl_handle) {
-		fprintf(stderr, "Failed to allocate netlink handle.\n");
+	state->nl_sock = nl_socket_alloc();
+	if (!state->nl_sock) {
+		fprintf(stderr, "Failed to allocate netlink sock.\n");
 		return -ENOMEM;
 	}
 
-	if (genl_connect(state->nl_handle)) {
+	if (genl_connect(state->nl_sock)) {
 		fprintf(stderr, "Failed to connect to generic netlink.\n");
 		err = -ENOLINK;
-		goto out_handle_destroy;
+		goto out_sock_destroy;
 	}
 
-	state->nl_cache = genl_ctrl_alloc_cache(state->nl_handle);
-	if (!state->nl_cache) {
+	if (genl_ctrl_alloc_cache(state->nl_sock, &state->nl_cache)) {
 		fprintf(stderr, "Failed to allocate generic netlink cache.\n");
 		err = -ENOMEM;
-		goto out_handle_destroy;
+		goto out_sock_destroy;
 	}
 
 	state->nl80211 = genl_ctrl_search_by_name(state->nl_cache, "nl80211");
@@ -61,8 +85,8 @@ static int nl80211_init(struct nl80211_state *state)
 
  out_cache_free:
 	nl_cache_free(state->nl_cache);
- out_handle_destroy:
-	nl_handle_destroy(state->nl_handle);
+ out_sock_destroy:
+	nl_socket_free(state->nl_sock);
 	return err;
 }
 
@@ -70,7 +94,7 @@ static void nl80211_cleanup(struct nl80211_state *state)
 {
 	genl_family_put(state->nl80211);
 	nl_cache_free(state->nl_cache);
-	nl_handle_destroy(state->nl_handle);
+	nl_socket_free(state->nl_sock);
 }
 
 static int reg_handler(struct nl_msg __attribute__((unused)) *msg,
@@ -119,7 +143,7 @@ nla_put_failure:
 
 int main(int argc, char **argv)
 {
-	int fd;
+	int fd = -1;
 	struct stat stat;
 	__u8 *db;
 	struct regdb_file_header *header;
@@ -138,7 +162,13 @@ int main(int argc, char **argv)
 	struct nlattr *nl_reg_rules;
 	int num_rules;
 
-	const char regdb[] = "/usr/lib/crda/regulatory.bin";
+	const char *regdb_paths[] = {
+		"/usr/local/lib/crda/regulatory.bin", /* Users/preloads can override */
+		"/usr/lib/crda/regulatory.bin", /* General distribution package usage */
+		"/lib/crda/regulatory.bin", /* alternative for distributions */
+		NULL
+	};
+	const char **regdb = regdb_paths;
 
 	if (argc != 1) {
 		fprintf(stderr, "Usage: %s\n", argv[0]);
@@ -159,7 +189,12 @@ int main(int argc, char **argv)
 
 	memcpy(alpha2, env_country, 2);
 
-	fd = open(regdb, O_RDONLY);
+	while (*regdb != NULL) {
+		fd = open(*regdb, O_RDONLY);
+		if (fd >= 0)
+			break;
+		regdb++;
+	}
 	if (fd < 0) {
 		perror("failed to open db file");
 		return -ENOENT;
@@ -271,7 +306,7 @@ int main(int argc, char **argv)
 	if (!cb)
 		goto cb_out;
 
-	r = nl_send_auto_complete(nlstate.nl_handle, msg);
+	r = nl_send_auto_complete(nlstate.nl_sock, msg);
 
 	if (r < 0) {
 		fprintf(stderr, "Failed to send regulatory request: %d\n", r);
@@ -283,7 +318,7 @@ int main(int argc, char **argv)
 	nl_cb_err(cb, NL_CB_CUSTOM, error_handler, NULL);
 
 	if (!finished) {
-		r = nl_wait_for_ack(nlstate.nl_handle);
+		r = nl_wait_for_ack(nlstate.nl_sock);
 		if (r < 0) {
 			fprintf(stderr, "Failed to set regulatory domain: "
 				"%d\n", r);
